var _class, _temp;

call((_temp = _class = function _class() {
  babelHelpers.classCallCheck(this, _class);
}, Object.defineProperty(_class, "test", {
  enumerable: true,
  writable: true,
  value: true
}), _temp));

var _class2 = function _class2() {
  babelHelpers.classCallCheck(this, _class2);
};

Object.defineProperty(_class2, "test", {
  enumerable: true,
  writable: true,
  value: true
});
export default _class2;
;
