"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _callee;

var _marked = /*#__PURE__*/regeneratorRuntime.mark(_callee);

function _callee() {
  var x;
  return regeneratorRuntime.wrap(function _callee$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          _context.next = 2;
          return 5;

        case 2:
          x = _context.sent;
          return _context.abrupt("return", 5);

        case 4:
        case "end":
          return _context.stop();
      }
    }
  }, _marked, this);
}
