var Foo = function () {
  function Foo() {
    babelHelpers.classCallCheck(this, Foo);
  }

  babelHelpers.createClass(Foo, [{
    key: "foo",
    value: function foo() {}
  }, {
    key: "foo",
    value: function foo() {}
  }, {
    key: bar,
    value: function value() {}
  }, {
    key: bar + "foo",
    value: function value() {}
  }]);
  return Foo;
}();
