"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _reactBootstrap = require("react-bootstrap");

exports.default = CustomModal = () => babelHelpers.jsx(_reactBootstrap.Modal.Header, {}, void 0, "foobar");