function foo(numVal /*:: ?*/) {}
function foo(numVal /*:: ?*/ = 2) {}
function foo(numVal /*: number*/) {}
function foo(numVal /*:: ?: number*/) {}
function foo(numVal /*: number*/ = 2) {}
function foo(numVal /*:: ?: number*/ = 2) {}