var _class, _temp;

var Foo = (_temp = _class = function Foo() {
  babelHelpers.classCallCheck(this, Foo);
}, Object.defineProperty(_class, "num", {
  enumerable: true,
  writable: true,
  value: 0
}), _temp);
