var rect = {};

var _rect$topLeft = babelHelpers.slicedToArray(rect.topLeft, 2),
    x1 = _rect$topLeft[0],
    y1 = _rect$topLeft[1],
    _rect$bottomRight = babelHelpers.slicedToArray(rect.bottomRight, 2),
    x2 = _rect$bottomRight[0],
    y2 = _rect$bottomRight[1];
