export type GraphQLFormattedError = number;
export type GraphQLFormattedError = {
  message: string,
  locations?: Array<{
    line: number,
    column: number
  }>
};
export interface foo { p: number }
export interface foo<T> { p: T }
