var A = function () {
  function A() {
    babelHelpers.classCallCheck(this, A);
  }

  babelHelpers.createClass(A, null, [{
    key: "a",
    value: function a() {}
  }, {
    key: "b",
    get: function get() {},
    set: function set(b) {}
  }]);
  return A;
}();
