var _ref = ["foo", "hello", [", ", "junk"], ["world"]],
    a = _ref[1],
    _ref$ = babelHelpers.slicedToArray(_ref[2], 1),
    b = _ref$[0],
    _ref$2 = babelHelpers.slicedToArray(_ref[3], 1),
    c = _ref$2[0],
    d = _ref[4];
