define(["exports"], function (exports) {
  "use strict";

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  exports.default = function (_a) {
    return {
      a: function a() {
        return _a;
      }
    };
  };
});
