import React from "react";

const Parent = ({}) => _ref;

export default Parent;

let _ref2 = <div className="child">
    ChildTextContent
  </div>,
    Child = () => _ref2,
    _ref = <div className="parent">
    <Child />
  </div>;
