var MyCtrl = function MyCtrl(a) {
  "any directive prologue";

  babelHelpers.classCallCheck(this, MyCtrl);
  foo;
};

var MyCtrl2 = function MyCtrl2(a) {
  "a";
  "b";

  babelHelpers.classCallCheck(this, MyCtrl2);
  foo;
};

var MyCtrl3 = function MyCtrl3(a) {
  "a";

  babelHelpers.classCallCheck(this, MyCtrl3);
  foo;
  "b";
};
