var rect = {};
var _rect$topLeft = rect.topLeft,
    x1 = _rect$topLeft.x,
    y1 = _rect$topLeft.y,
    _rect$bottomRight = rect.bottomRight,
    x2 = _rect$bottomRight.x,
    y2 = _rect$bottomRight.y;
var _ref = [0, 1, 2, 3, 4, 5, 6],
    foo = _ref[3],
    bar = _ref[5];
