"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
function _default() {
  return function () {
    function Select() {
      babelHelpers.classCallCheck(this, Select);
    }

    babelHelpers.createClass(Select, [{
      key: "query",
      value: function query(_query) {}
    }]);
    return Select;
  }();
}
exports.default = _default;
