var x = React.createElement(
  "div",
  null,
  React.createElement(Component, null)
);

var x = React.createElement(
  "div",
  null,
  props.children
);

var x = React.createElement(
  Composite,
  null,
  props.children
);

var x = React.createElement(
  Composite,
  null,
  React.createElement(Composite2, null)
);
