var Test = function () {
  function Test() {
    babelHelpers.classCallCheck(this, Test);
  }

  babelHelpers.createClass(Test, [{
    key: "test",
    set: function set(val) {
      this._test = val;
    }
  }]);
  return Test;
}();
