let a = true;
let b = false;
class e {}
function f() {}

switch (a) {
  case true:
    let c = 2;
    let foo = true;
    break;
  case false:
    let d = 3;
    foo = false;
    break;
}

switch (true) {
  case true:
    let a = false;
    let b = true;
    let c = 4;
    let d = 5;

  case false:
    class e {}
    function f() {}
}
