var a = 1;

for (var _a = 2, i = _a + 1; i < 10; i++) {
  console.log(i);
}

for (var _a2 in [1, 2, 3]) {
  var _a3 = 3;
  console.log(_a3);
}

try {
  throw new Error();
} catch (_a4) {
  var _a5 = 4;
}