import _Map from "babel-runtime/core-js/map";
import _Symbol from "babel-runtime/core-js/symbol";
import _Promise from "babel-runtime/core-js/promise";
obj.constructor === Object;
obj.constructor === _Promise;

_Symbol();
_Symbol("test");

new _Map();
