var t = function () {
  var e = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : "foo";
  var f = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 5;

  return e + " bar " + f;
};

var a = function (e) {
  var f = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 5;

  return e + " bar " + f;
};