function foo(numVal) {}
function foo(numVal) {}
function foo(numVal, strVal) {}
function foo(numVal, untypedVal) {}
function foo(untypedVal, numVal) {}
function foo(nullableNum) {}
function foo(callback) {}
function foo(callback) {}
function foo(callback) {}
function foo(callback) {}
function foo(callback) {}
function foo() {}
function foo() {}
function foo() {}
function foo() {}
function foo() {}
function foo() {}
function foo() {}
a = function () {};
a = { set fooProp(value) {} };
a = { set fooProp(value) {} };
a = { get fooProp() {} };
a = { id(x) {} };
a = { *id(x) {} };
a = { async id(x) {} };
a = { 123(x) {} };
class Foo {
  set fooProp(value) {}
}
class Foo2 {
  set fooProp(value) {}
}
class Foo3 {
  get fooProp() {}
}
var numVal;
var numVal = otherNumVal;
var a;
var a;
var a;
var a;
var a;
var a;
var a;
var a;
var a;
var a;
var a;
var a;
var a;
var a = [1, 2, 3];
a = class Foo {};
a = class Foo extends Bar {};
class Foo4 {}
class Foo5 extends Bar {}
class Foo6 extends mixin(Bar) {}
class Foo7 {
  bar() {
    return 42;
  }
}
class Foo8 {
  "bar"() {}
}
function foo(requiredParam, optParam) {}
class Foo9 {}
class Foo10 {}
var x = 4;
class Array {
  concat(items) {}
}
var x = fn;
var x = Y;
var x = Y;
var { x } = { x: "hello" };
var { x } = { x: "hello" };
var [x] = ["hello"];
function foo({ x }) {}
function foo([x]) {}
function foo(...rest) {}
(function (...rest) {});
(...rest) => rest;
var a;
var a;
var a;
var a;
var a;
var a;
var identity;
var identity;

import type from "foo";
import type2, { foo3 } from "bar";

import "foo";
import { V1 } from "foo";
import "foo";
import { V4 } from "foo";
