var t = function (f) {
  var _ref;

  arguments.length <= 1 ? undefined : arguments[1];
  _ref = (arguments.length <= 1 ? 0 : arguments.length - 1) - 1 + 1, arguments.length <= _ref ? undefined : arguments[_ref];
};

function t(f) {
  for (var _len = arguments.length, items = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    items[_key - 1] = arguments[_key];
  }

  items;
  items[0];
  items[items.length - 1];
}
