var _obj;

var obj = (_obj = {}, babelHelpers.defineProperty(_obj, foobar, function () {
  return "foobar";
}), babelHelpers.defineProperty(_obj, "test", function () {
  return "regular method after computed property";
}), _obj);
