var Ref = function Ref() {
  var ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : Ref;
  babelHelpers.classCallCheck(this, Ref);

  this.ref = ref;
};

var X = function X() {
  var x = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : foo;
  babelHelpers.classCallCheck(this, X);

  this.x = x;
};
