import React, {Component} from 'react';

export default class RandomComponent extends Component {
  constructor(){
    super();
  }

  render() {
    return (
      <div className='sui-RandomComponent'>
        <h2>Hi there!</h2>
      </div>
    );
  }
}
