let foo;
var foo;

let foo = "foo";
var foo = "bar";
const foo = "foo";

let foo,
    bar = "bar";
var foo,
    bar = "bar";

let foo = "foo",
    bar = "bar";
var foo = "foo",
    bar = "bar";
const foo = "foo",
      bar = "bar";
