"use strict";

var _taggedTemplateLiteral2 = require("babel-runtime/helpers/taggedTemplateLiteral");

var _taggedTemplateLiteral3 = _interopRequireDefault(_taggedTemplateLiteral2);

var _templateObject = (0, _taggedTemplateLiteral3.default)(["foo"], ["foo"]);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

tag(_templateObject);
