var a = true;
var b = false;
class e {}
function f() {}

switch (a) {
  case true:
    var c = 2;
    var foo = true;
    break;
  case false:
    var d = 3;
    foo = false;
    break;
}

switch (true) {
  case true:
    var _a = false;
    var _b = true;
    var _c = 4;
    var _d = 5;

  case false:
    class _e {}

    var _f = function () {};

}
