var BaseController = function (_Chaplin$Controller) {
  babelHelpers.inherits(BaseController, _Chaplin$Controller);

  function BaseController() {
    babelHelpers.classCallCheck(this, BaseController);
    return babelHelpers.possibleConstructorReturn(this, _Chaplin$Controller.apply(this, arguments));
  }

  return BaseController;
}(Chaplin.Controller);

var BaseController2 = function (_Chaplin$Controller$A) {
  babelHelpers.inherits(BaseController2, _Chaplin$Controller$A);

  function BaseController2() {
    babelHelpers.classCallCheck(this, BaseController2);
    return babelHelpers.possibleConstructorReturn(this, _Chaplin$Controller$A.apply(this, arguments));
  }

  return BaseController2;
}(Chaplin.Controller.Another);
