"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var b = function b() {
  babelHelpers.classCallCheck(this, b);
};

var a1 = function (_b) {
  babelHelpers.inherits(a1, _b);

  function a1() {
    babelHelpers.classCallCheck(this, a1);

    var _this = babelHelpers.possibleConstructorReturn(this, (a1.__proto__ || Object.getPrototypeOf(a1)).call(this));

    _this.x = function () {
      return _this;
    };
    return _this;
  }

  return a1;
}(b);

var a2 = function (_b2) {
  babelHelpers.inherits(a2, _b2);

  function a2() {
    babelHelpers.classCallCheck(this, a2);

    var _this2 = babelHelpers.possibleConstructorReturn(this, (a2.__proto__ || Object.getPrototypeOf(a2)).call(this));

    _this2.x = function () {
      return _this2;
    };
    return _this2;
  }

  return a2;
}(b);

exports.default = a2;
