System.register(["foo"], function (_export, _context) {
  "use strict";

  var foo, foo2;
  return {
    setters: [function (_foo) {
      foo = _foo.default;
      foo2 = _foo.default;
    }],
    execute: function () {}
  };
});