"use strict";

var A = function A() {
  babelHelpers.classCallCheck(this, A);
};

var B = function (_A) {
  babelHelpers.inherits(B, _A);

  function B() {
    var _this, _ret;

    babelHelpers.classCallCheck(this, B);

    return _ret = (_this = babelHelpers.possibleConstructorReturn(this, (B.__proto__ || Object.getPrototypeOf(B)).call(this)), _this), babelHelpers.possibleConstructorReturn(_this, _ret);
  }

  return B;
}(A);
