var _ref = <span>Sub Component</span>;

class Component extends React.Component {
  constructor(...args) {
    var _temp;

    var _ref2 = <this.subComponent />;

    return _temp = super(...args), this.subComponent = () => _ref, this.render = () => _ref2, _temp;
  }

}
