"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
var foo = exports.foo = function foo(gen) {
  var ctx = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
};

var bar = exports.bar = function bar(gen) {
  var ctx = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
};