var x = React.createElement(
  "div",
  null,
  React.createElement(
    "div",
    null,
    React.createElement("br", null)
  ),
  React.createElement(
    Component,
    null,
    foo,
    React.createElement("br", null),
    bar
  ),
  React.createElement("br", null)
);
