// @flow
var C = function () {
  function C() {
    babelHelpers.classCallCheck(this, C);
  }

  babelHelpers.createClass(C, [{
    key: 'm',
    value: function m(x: number): string {
      return 'a';
    }
  }]);
  return C;
}();
