var func = /*#__PURE__*/regeneratorRuntime.mark(function _callee() {var actual;return regeneratorRuntime.wrap(function _callee$(_context) {while (1) switch (_context.prev = _context.next) {case 0:_context.next = 2;return (
                    obj.
                    method().
                    method2());case 2:

                actual = true;case 3:case "end":return _context.stop();}}, _callee, this);});
