function foo(bar = "bar") {}

function foo(bar = "bar", ...items) {}

function foo(...items) {}

function* foo() {}

function foo() {}

async function foo() {}

var foo = function* foo() {};

var foo = function foo() {};

var foo = async function foo() {};

var foo = function* () {};

var foo = function () {};

var foo = async function () {};
