'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _foo = require('foo');

var _foo2 = _interopRequireDefault(_foo);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

console.log(_foo2['default']);

exports['default'] = 5;
