export default class App extends React.Component {
  render() {
    return _ref;
  }
}

const _ref2 = <div>child</div>,
      AppItem = () => {
  return _ref2;
},
      _ref = <div>
        <p>Parent</p>
        <AppItem />
      </div>;