var Test = function (_Foo) {
  babelHelpers.inherits(Test, _Foo);

  function Test() {
    babelHelpers.classCallCheck(this, Test);
    return babelHelpers.possibleConstructorReturn(this, _Foo.apply(this, arguments));
  }

  return Test;
}(Foo);
