module.exports = function preset (context, options, fileContext) {
  if (/resolve-addons-relative-to-file$/.test(fileContext.dirname)) {
    return {
      plugins: [plugin],
    };
  }
  return {};
};

function plugin () {
  return {
    visitor: {
      StringLiteral: function (path) {
        path.node.value =
          path.node.value.toUpperCase();
      },
    },
  };
}
