export var foo = "yes", foob = "no";

export function whatever() {}

export default function wowzers() {}

var bar = {
  foo: function () {
    foo;
  },

  whatever: function () {
    whatever;
  },

  wowzers: function () {
    wowzers;
  }
};
