var Test = function Test() {
  babelHelpers.classCallCheck(this, Test);

  babelHelpers.get(Test.prototype.__proto__ || Object.getPrototypeOf(Test.prototype), "hasOwnProperty", this).call(this, "test");
};
