var x = (React.createElement("div", null,
  /* A comment at the beginning */
  /* A second comment at the beginning */
  React.createElement("span", null
    /* A nested comment */
  ),
  /* A sandwiched comment */
  React.createElement("br", null)
  /* A comment at the end */
  /* A second comment at the end */
));
