"use strict";

var _a2 = require("./a");

var Foo = function () {
  function Foo() {
    babelHelpers.classCallCheck(this, Foo);
  }

  babelHelpers.createClass(Foo, [{
    key: "_a",
    value: function _a() {
      (0, _a2.a)();
    }
  }]);
  return Foo;
}();
