// from code:
// export default {
//   plugins: [
//     require('../../../../../babel-plugin-syntax-decorators'),
//   ]
// };
'use strict';

exports.__esModule = true;
exports.default = {
  plugins: [require('../../../../../babel-plugin-syntax-decorators')]
};
