System.register([], function (_export, _context) {
  "use strict";

  return {
    setters: [],
    execute: function () {
      _export("default", function (_a) {
        return {
          a: function a() {
            return _a;
          }
        };
      });
    }
  };
});
